// mfcView.cpp : implementation of the CMfcView class
//

#include "stdafx.h"
#include "mfc.h"

#include "mfcDoc.h"
#include "mfcView.h"
#include "transformdlg.h"
#ifndef _CANVAS
#include "canvas.hpp"
#endif

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMfcView

IMPLEMENT_DYNCREATE(CMfcView, CView)

BEGIN_MESSAGE_MAP(CMfcView, CView)
	//{{AFX_MSG_MAP(CMfcView)
	ON_COMMAND(ID_TRANSFORM_ROTATE, OnTransformRotate)
	ON_COMMAND(ID_TRANSFORM_SCALE, OnTransformScale)
	ON_COMMAND(ID_TRANSFORM_TRANSLATE, OnTransformTranslate)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_TIMER()
	ON_COMMAND(ID_TRANSFORM_ANIMATE, OnTransformAnimate)
	ON_COMMAND(ID_OBJECTS_SPHERE, OnObjectsSphere)
	ON_UPDATE_COMMAND_UI(ID_OBJECTS_SPHERE, OnUpdateObjectsSphere)
	ON_COMMAND(ID_OBJECTS_CONE, OnObjectsCone)
	ON_UPDATE_COMMAND_UI(ID_OBJECTS_CONE, OnUpdateObjectsCone)
	ON_COMMAND(ID_OBJECTS_CYLINDER, OnObjectsCylinder)
	ON_UPDATE_COMMAND_UI(ID_OBJECTS_CYLINDER, OnUpdateObjectsCylinder)
	//}}AFX_MSG_MAP
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMfcView construction/destruction

CMfcView::CMfcView()
{
	// TODO: add construction code here
	m_xRotate=0.0f;
	m_yRotate=0.0f;
	m_zRotate=0.0f;
	m_animate=FALSE;
	m_yaw=0.0f;
	m_xTranslate=0.0f;
	m_yTranslate=0.0f;
	m_zTranslate=0.0f;
	m_xScale=1.0f;
	m_yScale=1.0f;
	m_zScale=1.0f;
	m_cone_flag=FALSE;
	m_cylinder_flag=FALSE;
	m_sphere_flag=FALSE;
}

CMfcView::~CMfcView()
{
}

BOOL CMfcView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	cs.style |= WS_CLIPCHILDREN | WS_CLIPSIBLINGS;

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CMfcView drawing

void CMfcView::OnDraw(CDC* pDC)
{
	CMfcDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// TODO: add draw code for native data here
	if(m_hPalette)
	{
		SelectPalette(pDC->m_hDC,m_hPalette,FALSE);
		RealizePalette(pDC->m_hDC);
	}
	wglMakeCurrent(pDC->m_hDC,m_hRC);
	DrawWithOpenGL();
	SwapBuffers(pDC->m_hDC);
	wglMakeCurrent(pDC->m_hDC,NULL);
}

/////////////////////////////////////////////////////////////////////////////
// CMfcView printing

BOOL CMfcView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CMfcView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CMfcView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CMfcView diagnostics

#ifdef _DEBUG
void CMfcView::AssertValid() const
{
	CView::AssertValid();
}

void CMfcView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CMfcDoc* CMfcView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CMfcDoc)));
	return (CMfcDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMfcView message handlers

void CMfcView::OnTransformRotate() 
{
	// TODO: Add your command handler code here
	CTransformDlg dlg;
	
	dlg.m_xAxis=m_xRotate;
	dlg.m_yAxis=m_yRotate;
	dlg.m_zAxis=m_zRotate;

	int response=dlg.DoModal();

	if(response==IDOK)
	{
		m_xRotate=(GLfloat) dlg.m_xAxis;
		m_yRotate=(GLfloat) dlg.m_yAxis;
		m_zRotate=(GLfloat) dlg.m_zAxis;
	}
	Invalidate();
}

void CMfcView::OnTransformScale() 
{
	// TODO: Add your command handler code here

	CTransformDlg dlg;
	
	dlg.m_xAxis=m_xScale;
	dlg.m_yAxis=m_yScale;
	dlg.m_zAxis=m_zScale;

	int response=dlg.DoModal();

	if(response==IDOK)
	{
		m_xScale=(GLfloat) dlg.m_xAxis;
		m_yScale=(GLfloat) dlg.m_yAxis;
		m_zScale=(GLfloat) dlg.m_zAxis;
	}

}

void CMfcView::OnTransformTranslate() 
{
	// TODO: Add your command handler code here
		CTransformDlg dlg;
	
	dlg.m_xAxis=m_xTranslate;
	dlg.m_yAxis=m_yTranslate;
	dlg.m_zAxis=m_zTranslate;

	int response=dlg.DoModal();

	if(response==IDOK)
	{
		m_xTranslate=(GLfloat) dlg.m_xAxis;
		m_yTranslate=(GLfloat) dlg.m_yAxis;
		m_zTranslate=(GLfloat) dlg.m_zAxis;
	}

}


int CMfcView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	

	PIXELFORMATDESCRIPTOR pixelDesc=
	{
		sizeof(PIXELFORMATDESCRIPTOR),
			1,
			PFD_DRAW_TO_WINDOW |
			PFD_SUPPORT_OPENGL |
			PFD_DOUBLEBUFFER,
			PFD_TYPE_RGBA,
			24,
			0,0,0,0,0,0,
			0,0,0,0,0,0,0,
			32,
			0,0,
			PFD_MAIN_PLANE,
			0,
			0,0,0
	};


	CClientDC clientDC(this);

	m_GLPixelIndex = ChoosePixelFormat(clientDC.m_hDC, &pixelDesc);

	SetPixelFormat( clientDC.m_hDC, 
				  m_GLPixelIndex, 
				  &pixelDesc);
	
	DescribePixelFormat(clientDC.m_hDC, m_GLPixelIndex, sizeof(pixelDesc), &pixelDesc);

	if(pixelDesc.dwFlags & PFD_NEED_PALETTE)
		setupLogicalPalette();

	m_hRC=wglCreateContext(clientDC.m_hDC);

	return 0;
}

void CMfcView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here
	wglDeleteContext(m_hRC);
	if(m_hPalette)
	{
		DeleteObject(m_hPalette);
	}
}
void CMfcView::DrawWithOpenGL()
{
	static int spin=0;

	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glClearColor(1.0f,0.0f,0.0f,1.0f);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	m_canvas.setLighting();

	setDepthBuffer();

	glShadeModel(GL_SMOOTH);

	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	m_canvas.copperMaterial();

	glPushMatrix();
		glTranslatef(0.0f,5.0f,-20.0f);
		m_canvas.sphere(2.5f);
	glPopMatrix();

	m_canvas.brassMaterial();

	glPushMatrix();
		glTranslatef(0.0f,0.0f,-20.0f);
		glScalef(m_xScale,m_yScale,m_zScale);
		glPushMatrix();
			glTranslatef(m_xTranslate,m_yTranslate,m_zTranslate);
			glRotatef(m_xRotate,1.0f,0.0f,0.0f);
			glRotatef(m_yRotate,0.0f,1.0f,0.0f);
			glRotatef(m_zRotate,0.0f,0.0f,1.0f);
			glRotatef(m_yaw,0.0f,1.0f,0.0f);
			processGeometry();
		glPopMatrix();
	glPopMatrix();

	glFlush();
}
void CMfcView::processGeometry()
{
	if (m_cone_flag==TRUE)
	{
		m_canvas.cone(3.0f,3.0f);
	}
	else if(m_cylinder_flag==TRUE)
	{
		m_canvas.cylinder(5,5);
	}
	else if(m_sphere_flag==TRUE)
	{
		m_canvas.sphere(5.0f);
	}
	else
	{
		m_canvas.box(2.5f,2.5f,2.5f);
	}
}
void CMfcView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	GLdouble aspect;
	
	// TODO: Add your message handler code here

	if (cy==0)
		aspect = (GLdouble)cx;
	else
		aspect = (GLdouble)cx/(GLdouble)cy;

	CClientDC clientDC(this);
	wglMakeCurrent(clientDC,m_hRC);

	setPerspectiveWindow(60,aspect,5,4000);
	setViewport(0,0,cx,cy);

	wglMakeCurrent(NULL,NULL);


}

void CMfcView::setPerspectiveWindow(GLdouble fovy, GLdouble aspect,GLdouble zNear,GLdouble zFar)
{
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(fovy, aspect,zNear,  zFar);
}
void CMfcView::setViewport(GLint iX, GLint iY, GLsizei iWidth, GLsizei iHeight)
{
	glViewport(iX,iY,iWidth,iHeight);
}
void CMfcView::setDepthBuffer()
{
  glEnable(GL_DEPTH_TEST);
  glDepthFunc(GL_LESS);
  glEnable(GL_CULL_FACE);
  glCullFace(GL_BACK);
  glFrontFace(GL_CCW);
}
void CMfcView::setupLogicalPalette()
{
	struct
	{
		WORD Version;
		WORD NumberOfEntries;
		PALETTEENTRY aEntries[256];
	}logicalPalette={0x300,256};

	BYTE reds[]={0,36,72,109,145,182,218,255};
	BYTE greens[]={0,36,72,109,145,182,218,255};
	BYTE blues[]={0,85,170,255};

	for(int colorNum=0; colorNum<256; ++colorNum)
	{
		logicalPalette.aEntries[colorNum].peRed= reds[colorNum&0x07];
		logicalPalette.aEntries[colorNum].peGreen=greens[(colorNum>>0x03) &0x07];
		logicalPalette.aEntries[colorNum].peBlue=blues[(colorNum>>0x06) &0x03];
		logicalPalette.aEntries[colorNum].peFlags=0;
	}
	m_hPalette=CreatePalette((LOGPALETTE*)&logicalPalette);
}
void CMfcView::OnTimer(UINT nIDEvent) 
{
	// TODO: Add your message handler code here and/or call default
	m_yaw=(GLfloat)((GLint)(m_yaw+5.0f)%360);
	Invalidate(FALSE);
	CView::OnTimer(nIDEvent);
}

void CMfcView::OnTransformAnimate() 
{
	// TODO: Add your command handler code here
	
	if(m_animate)
	{
		KillTimer(1);
		m_animate=FALSE;
	}
	else
	{
		SetTimer(1,1,NULL);
		m_animate=TRUE;
	}
}

void CMfcView::OnObjectsSphere() 
{
	// TODO: Add your command handler code here
	if(m_sphere_flag==FALSE)
	{
		m_sphere_flag=TRUE;
	}
	else
	{
		m_sphere_flag=FALSE;
	}
	Invalidate();
	
}

void CMfcView::OnUpdateObjectsSphere(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if (m_sphere_flag==TRUE)
	{
		pCmdUI->SetCheck(TRUE);
	}
	else
	{
		pCmdUI->SetCheck(FALSE);
	}
	
}

void CMfcView::OnObjectsCone() 
{
	// TODO: Add your command handler code here
	if(m_cone_flag==FALSE)
	{
		m_cone_flag=TRUE;
	}
	else
	{
		m_cone_flag=FALSE;
	}
	Invalidate();
}

void CMfcView::OnUpdateObjectsCone(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if (m_cone_flag==TRUE)
	{
		pCmdUI->SetCheck(TRUE);
	}
	else
	{
		pCmdUI->SetCheck(FALSE);
	}
	
}

void CMfcView::OnObjectsCylinder() 
{
	// TODO: Add your command handler code here
	if(m_cylinder_flag==FALSE)
	{
		m_cylinder_flag=TRUE;
	}
	else
	{
		m_cylinder_flag=FALSE;
	}
	Invalidate();
	
}

void CMfcView::OnUpdateObjectsCylinder(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	if (m_cylinder_flag==TRUE)
	{
		pCmdUI->SetCheck(TRUE);
	}
	else
	{
		pCmdUI->SetCheck(FALSE);
	}
	
}
